# @Author: yican.kz
# @Date: 2019-08-02 11:48:26
# @Last Modified by:   yican.kz
# @Last Modified time: 2019-08-02 11:48:26

from . import nn
from . import utils

name = "torch_buddy"
__author__ = "鲲(China)"
__version__ = "0.0.5"

__all__ = ["utils", "nn"]



# @Author: yican.kz
# @Date: 2019-08-02 11:40:56
# @Last Modified by:   yican.kz
# @Last Modified time: 2019-08-02 11:40:56

import os
import sys
import time
import glob

import numpy as np
import pandas as pd
from IPython.display import display
from sklearn.externals import joblib


# ==============================================================================
# 时间相关
# ==============================================================================
class tick_tock:
    def __init__(self, process_name, verbose=1):
        self.process_name = process_name
        self.verbose = verbose

    def __enter__(self):
        if self.verbose:
            print(self.process_name + " begin ......")
            self.begin_time = time.time()

    def __exit__(self, type, value, traceback):
        if self.verbose:
            end_time = time.time()
            print(self.process_name + " end ......")
            print("time lapsing {0} s \n".format(end_time - self.begin_time))


# ==============================================================================
# 文件相关
# ==============================================================================


def mkdir(path: str):
    """Create directory.

     Create directory if it is not exist, else do nothing.

     Parameters
     ----------
     path: str
        Path of your directory.

     Examples
     --------
     mkdir("data/raw/train")
     """
    try:
        os.stat(path)
    except Exception:
        os.makedirs(path)


def remove_temporary_files(folder_path: str):
    """Remove files begin with ".~".

     Parameters
     ----------
     folder_path: str
        Folder path which you want to clean.

     Examples
     --------
     remove_temporary_files("data/raw/")

    """
    num_of_removed_file = 0
    for fname in os.listdir(folder_path):
        if fname.startswith("~") or fname.startswith("."):
            num_of_removed_file += 1
            os.remove(folder_path + "/" + fname)
    print("{0} file have been removed".format(num_of_removed_file))


def remove_all_files(folder_path: str):
    """Remove all files under folder_path.

     Parameters
     ----------
     folder_path: str
        Folder path which you want to clean.

     Examples
     --------
     remove_all_files("data/raw/")

    """
    folder = folder_path + "*"
    files = glob.glob(folder)
    for file in files:
        os.remove(file)


def save_last_n_files(directory, max_to_keep=10, suffix="*.p"):
    """Save max_to_keep files with suffix in directory

     Parameters
     ----------
     directory: str
        Folder path which you save files.
     max_to_keep: int
        Maximum files to keep.
     suffix: str
        File format.

     Examples
     --------
     save_last_n_files("data/raw/")
    """
    saved_model_files = glob.glob(directory + suffix)
    saved_model_files_lasted_n = sorted(saved_model_files, key=os.path.getctime)[-max_to_keep:]
    files_tobe_deleted = set(saved_model_files).difference(saved_model_files_lasted_n)

    for file in files_tobe_deleted:
        os.remove(file)


def restore(directory, suffix="*.p", filename=None):
    """Restore model from file directory.

     Parameters
     ----------
     directory: str
        Folder path which you save files.
     filename: str
        Filename you want to restore.
     suffix: str
        File format.

     Examples
     --------
     restore("data/raw/")
    """
    # If model_file is None, restore the newest one, else restore the specified one.
    if filename is None:
        filename = sorted(glob.glob(directory + suffix), key=os.path.getctime)[-1]
    model = joblib.load(filename)
    print("Restore from file : {}".format(filename))
    return model, filename


# ==============================================================================
# 展示相关
# ==============================================================================
def display_pro(data: pd.DataFrame, n=5):
    """Pro version of display function.

     Display [memory usage], [data shape] and [first n rows] of a pandas dataframe.

     Parameters
     ----------
     data: pandas dataframe
        Pandas dataframe to be displayed.
     n: int
        First n rows to be displayed.

     Example
     -------
     import pandas as pd
     from sklearn.datasets import load_boston
     data = load_boston()
     data = pd.DataFrame(data.data)
     display_pro(data)

        Parameters
        ----------
        data: pandas dataframe


        Returns
        -------
            None
    """
    print("Data shape   : {}".format(data.shape))
    display(data[:n])


def memory_usage(data: pd.DataFrame, detail=1):
    """Show memory usage.

     Parameters
     ----------
     data: pandas dataframe
     detail: int, optinal (default = 1)
        0: show memory of each column
        1: show total memory

     Examples
     --------
     import pandas as pd
     from sklearn.datasets import load_boston
     data = load_boston()
     data = pd.DataFrame(data.data)
     memory = memory_usage(data)
     """

    memory_info = data.memory_usage()
    if detail:
        display(memory_info)

    if type(memory_info) == int:
        memory = memory_info / (1024 * 1024)
    else:
        memory = data.memory_usage().sum() / (1024 * 1024)
    print("Memory usage : {0:.2f}MB".format(memory))
    return memory


class ProgressBar:
    def __init__(self, n_batch, bar_len=80):
        """Brief description.

        Detailed description.

        Parameters
        ----------
        bar_len: int
            The length you want to display your bar.
        n_batch: int
            Total rounds to iterate.
        Returns
        -------
        None

        Examples
        --------
        import time
        progressBar = ProgressBar(100)

        for i in range(100):
            progressBar.step(i)
            time.sleep(0.1)
        """
        self.bar_len = bar_len
        self.progress_used = 0
        self.progress_remanent = bar_len
        self.n_batch = n_batch

    def step(self, i):
        self.progress_used = int(np.round(i * self.bar_len / self.n_batch))
        self.progress_remanent = self.bar_len - self.progress_used
        sys.stdout.write(
            "\r"
            + ">" * self.progress_used
            + "Epoch Progress: "
            + "{:.2%}".format((i) / self.n_batch)
            + "=" * self.progress_remanent
        )
        sys.stdout.flush()



# @Author: yican.kz
# @Date: 2019-08-02 11:40:53
# @Last Modified by:   yican.kz
# @Last Modified time: 2019-08-02 11:40:53


from .nn import word_idx, unicode_to_ascii
from .plot import subplots
from .image import show_image, draw_rect, draw_text
from .helper import (
    tick_tock,
    display,
    memory_usage,
    remove_temporary_files,
    save_last_n_files,
    display_pro,
    remove_all_files,
    ProgressBar,
    restore,
)

__all__ = [
    "word_idx",
    "unicode_to_ascii",
    "subplots",
    "show_image",
    "draw_rect",
    "draw_text",
    "tick_tock",
    "display",
    "memory_usage",
    "remove_temporary_files",
    "save_last_n_files",
    "display_pro",
    "remove_all_files",
    "ProgressBar",
    "restore",
]



# @Author: yican.kz
# @Date: 2019-08-02 11:41:11
# @Last Modified by:   yican.kz
# @Last Modified time: 2019-08-02 11:41:11

import numpy as np
import matplotlib

matplotlib.use("TkAgg")
import matplotlib.pyplot as plt


def subplots(data_for_plots, figsize=[12, 4]):
    """
        data_for_plots = [[1,2,3], [4,5,6]]
    """
    f, axes = plt.subplots(np.int(np.ceil(len(data_for_plots) / 2)), 2, figsize=figsize)
    for ax, data_for_plot in zip(axes.flat, data_for_plots):
        ax.plot(data_for_plot)



# @Author: yican.kz
# @Date: 2019-08-02 11:40:22
# @Last Modified by:   yican.kz
# @Last Modified time: 2019-08-02 11:40:22

from .base import PTBaseModel
from .base import PTBaseDataLoader

from .cv import (
    xyccwd_to_xymmmm,
    xymmmm_to_xyccwd,
    xyccwd_to_xygcgcgwgh,
    xygcgcgwgh_to_xyccwd,
    find_intersection,
    find_jaccard_overlap,
    expand,
    random_crop,
    flip,
    resize,
    photometric_distort,
    transform,
)
from .nn_helper import (
    AverageMeter,
    VisdomLinePlotter,
    pytorch_reproducer,
    get_device,
    clip_gradient,
    init_conv2d,
    decimate,
    adjust_learning_rate,
    get_learning_rate,
    LRFinder,
    StateCacher,
)
from .lr_scheduler import ConstantLR, LinearLR4Finder, ExponentialLR4Finder

__all__ = [
    "PTBaseModel",
    "PTBaseDataLoader",
    "ConstantLR",
    "LinearLR4Finder",
    "ExponentialLR4Finder",
    "AverageMeter",
    "VisdomLinePlotter",
    "pytorch_reproducer",
    "get_device",
    "clip_gradient",
    "init_conv2d",
    "decimate",
    "adjust_learning_rate",
    "get_learning_rate",
    "LRFinder",
    "StateCacher",
    "xyccwd_to_xymmmm",
    "xymmmm_to_xyccwd",
    "xyccwd_to_xygcgcgwgh",
    "xygcgcgwgh_to_xyccwd",
    "find_intersection",
    "find_jaccard_overlap",
    "expand",
    "randomrandom_crop",
    "flip",
    "resize",
    "photometric_distort",
    "transform",
]



# @Author: yican.kz
# @Date: 2019-08-02 11:41:02
# @Last Modified by:   yican.kz
# @Last Modified time: 2019-08-02 11:41:02

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont


def show_image(path: str):
    """展示图片，输入为图片路径
       Parameters
       ----------
       path: string
           image path

       Example
       -------
       img_path = "../data/raw/gamble/pictures/_更新页面_xdqp__仙豆棋牌__0.png"
       image = show_image(img_path)
    """
    image = Image.open(path, mode="r")
    image = image.convert("RGB")
    return image


def draw_rect(image: Image.Image, location_box: list, color: str):
    """画方框
       Parameters
       ----------
       image: string
           image path
       location_box: list
           [x_min, y_min, x_max, y_max]

       Example
       -------
       img_path = "../data/raw/gamble/pictures/_更新页面_xdqp__仙豆棋牌__0.png"
       image = show_image(img_path)
       location_box = [50, 50, 100, 100]
       image = draw_rect(image, location_box, 'red')
    """
    draw = ImageDraw.Draw(image)
    draw.rectangle(xy=location_box, outline=color)
    draw.rectangle(xy=[ll + 1 for ll in location_box], outline=color)
    return image


def draw_text(image: Image.Image, xy: list, color: str, text: str):
    """画文字
       Parameters
       ----------
       image: string
           image path

       Example
       -------
       img_path = "../data/raw/gamble/pictures/_更新页面_xdqp__仙豆棋牌__0.png"
       image = show_image(img_path)
       location_box = [50, 50, 100, 100]
       image = draw_rect(image, location_box, 'red')
    """
    font = ImageFont.truetype("../fonts/calibri/Calibri.ttf", 15)
    draw = ImageDraw.Draw(image)
    text_size = font.getsize(text)
    location_text = [xy[0] + 2.0, xy[1] - text_size[1]]
    location_textbox = [xy[0], xy[1] - text_size[1], xy[0] + text_size[0] + 4.0, xy[1]]
    draw.rectangle(xy=location_textbox, fill=color)
    draw.text(xy=location_text, text=text, fill="white", font=font)
    return image
